<?php

namespace TotalPoll\Modules\Extensions\Duplicate;
! defined( 'ABSPATH' ) && exit();


use TotalPollVendors\TotalCore\Helpers\Arrays;
use TotalPollVendors\TotalCore\Helpers\Misc;
use TotalPollVendors\TotalCore\Helpers\Strings;
use TotalPollVendors\TotalCore\Http\Request;

/**
 * Class Extension
 * @package TotalPoll\Modules\Extensions\Duplicate
 */
class Extension extends \TotalPoll\Modules\Extension {
	protected $root = __FILE__;
	/**
	 * @var bool $enqueue A flag to enqueue extension style.
	 */
	protected $enqueue = false;

	/**
	 * @var Request $request
	 */
	protected $request;

	/**
	 * Scopes.
	 */
	const ALL = 'all';
	const ONLY_DATA = 'data';
	const ONLY_SETTINGS = 'settings';

	/**
	 * Run the extension.
	 *
	 * @return mixed
	 */
	public function run() {
		// Dependencies
		$this->request = TotalPoll( 'http.request' );

		// Actions
		add_action( 'post_submitbox_misc_actions', [ $this, 'duplicateButton' ] );
		add_filter( 'totalpoll/filters/admin/listing/actions', [ $this, 'duplicateActions' ], 10, 2 );
		add_action( 'admin_action_duplicate_poll', [ $this, 'duplicatePoll' ] );
		add_action( 'admin_footer', [ $this, 'assets' ] );
	}

	/**
	 * Extension assets.
	 *
	 * @return void
	 */
	public function assets() {
		if ( $this->enqueue ):
			wp_enqueue_style( 'totalpoll-duplicate-extension', $this->getUrl( 'assets/styles/style.css' ) );
		endif;
	}

	/**
	 * Duplicate button.
	 *
	 * @return void
	 */
	public function duplicateButton() {
		$postId = $this->request['post'];

		if ( ! empty( $postId ) ):
			$post = get_post( $postId );

			if ( $post->post_type === TP_POLL_CPT_NAME && current_user_can( 'edit_poll', $post->ID ) ):
				$this->enqueue = true;
				echo $this->getDuplicateMenu( $post->ID );
			endif;
		endif;
	}

	/**
	 * Duplicate inline row actions.
	 *
	 * @param array    $actions
	 * @param \WP_Post $post
	 *
	 * @return array
	 */
	public function duplicateActions( $actions, $post ) {

		if ( current_user_can( 'edit_poll', $post->ID ) ):
			$this->enqueue   = true;
			$actions['copy'] = $this->getDuplicateMenu( $post->ID );
		endif;

		return $actions;
	}

	/**
	 * Duplicate poll.
	 *
	 * @return void
	 */
	public function duplicatePoll() {
		// Redirection
		$back = wp_get_referer();

		// Poll and scope
		$poll  = $this->request['poll'];
		$scope = $this->request['scope'];

		// Poll post
		$pollPost = get_post( $poll );

		// Check criteria
		if ( ! $pollPost || $pollPost->post_type !== TP_POLL_CPT_NAME || ! current_user_can( 'create_polls' ) || ! wp_verify_nonce( $this->request['_wpnonce'] ) ):
			wp_die( esc_html__( 'Invalid action call.', 'totalpoll' ) );
		endif;

		// Get current poll model
		$pollModel = (array) json_decode( $pollPost->post_content ?: '{}', true );

		// Get new poll model defaults
		$newPollModel = TotalPoll( 'polls.defaults', [] );

		// Flags
		$copyData     = $scope === self::ALL || $scope === self::ONLY_DATA;
		$copySettings = $scope === self::ALL || $scope === self::ONLY_SETTINGS;

		// Copy data
		if ( $copyData ):
			// Clone questions
			$newPollModel['questions'] = $pollModel['questions'];

			// Reset votes and regenerate uid
			foreach ( $newPollModel['questions'] as &$question ):
				$question['uid'] = Misc::generateUid();

				foreach ( $question['choices'] as &$choice ):
					$choice['uid']           = Misc::generateUid();
					$choice['votes']         = 0;
					$choice['votesOverride'] = 0;
				endforeach;

			endforeach;
		endif;

		// Copy settings
		if ( $copySettings ):
			// Remove questions
			unset( $pollModel['questions'] );
			// Copy settings (except questions)
			$newPollModel = Arrays::parse( $pollModel, $newPollModel );
		endif;

		// Prepare settings for insertion
		$newPollModel = json_encode( $newPollModel );

		// Sanitize
		if ( ! current_user_can( 'unfiltered_html' ) ):
			$newPollModel = wp_kses_post( $newPollModel );
		endif;

		// Add slashes
		$newPollModel = wp_slash( $newPollModel );

		// Insert cloned poll
		$newPoll = wp_insert_post(
			[
				'post_type'    => $pollPost->post_type,
				'post_title'   => Strings::template(
				 esc_html__( 'Copy of {{title}}', 'totalpoll' ),
					[ 'title' => $pollPost->post_title ]
				),
				'post_status'  => 'draft',
				'post_content' => $newPollModel,
			]
		);

		// Redirection target
		if ( is_wp_error( $newPoll ) ):
			$back = $back ?: admin_url( 'edit.php?post_type=' . TP_POLL_CPT_NAME );
		else:
			$back = admin_url( "post.php?post={$newPoll}&action=edit" );
		endif;

		wp_safe_redirect( $back );
		exit;
	}

	/**
	 * Get duplicate menu.
	 *
	 * @param $pollId
	 *
	 * @return string
	 */
	protected function getDuplicateMenu( $pollId ) {
		$actions = [
			[ 'label' => esc_html__( 'Duplicate', 'totalpoll' ), 'url' => $this->duplicateUrl( $pollId, self::ALL ) ],
			[ 'label' => esc_html__( 'Duplicate (Data)', 'totalpoll' ), 'url' => $this->duplicateUrl( $pollId, self::ONLY_DATA ) ],
			[ 'label' => esc_html__( 'Duplicate (Settings)', 'totalpoll' ), 'url' => $this->duplicateUrl( $pollId, self::ONLY_SETTINGS ) ],
		];

		ob_start();
		include 'views/menu.php';

		return ob_get_clean();
	}

	/**
	 * Get duplicate URL.
	 *
	 * @param integer $pollId
	 * @param string  $scope
	 *
	 * @return string
	 */
	protected function duplicateUrl( $pollId, $scope = self::ALL ) {
		return wp_nonce_url( admin_url( "admin.php?action=duplicate_poll&scope={$scope}&poll={$pollId}" ) );
	}
}
