<?php

namespace TotalPoll\Modules\Extensions\EmailCapture;
! defined( 'ABSPATH' ) && exit();


use TotalPoll\Modules\Extensions\EmailCapture\Contract\EmailService;
use TotalPoll\Modules\Extensions\EmailCapture\Services\AbstractService;
use TotalPoll\Modules\Extensions\EmailCapture\Services\MailChimp;
use TotalPoll\Poll\Model;
use TotalPollVendors\TotalCore\Helpers\Misc;

/**
 * Class Extension
 * @package TotalPoll\Modules\Extensions\EmailCapture
 */
class Extension extends \TotalPoll\Modules\Extension {

	const OPTION_NAME = 'emailcapture';

	/**
	 * @var string
	 */
	protected $root = __FILE__;

	/**
	 * @var EmailService[]
	 */
	protected $services = [];

	/**
	 * @var string
	 */
	protected $emailField = 'email';

	/**
	 * @var string
	 */
	protected $consentField = 'consent';

	/**
	 * Extension constructor.
	 *
	 * @param array $options
	 */
	public function __construct( $options = [] ) {
		parent::__construct( TotalPoll()->option( self::OPTION_NAME, [] ) );
	}

	/**
	 * @return mixed|void
	 */
	public function run() {
		add_filter( 'totalpoll/filters/admin/options/tabs', [ $this, 'tab' ] );
		add_filter( 'totalpoll/filters/admin/editor/settings/tabs', [ $this, 'editor' ] );
		add_filter( 'totalpoll/filters/poll/settings-item/fields', [ $this, 'fields' ] );
		add_filter( 'totalpoll/filters/form/fields', [ $this, 'formFields' ] );
		add_action( 'totalpoll/actions/after/admin/options/tabs/content', [ $this, 'render' ] );
		add_action( 'totalpoll/actions/after/poll/command/save-entry', [ $this, 'handle' ], 10, 2 );

		$this->services = apply_filters( 'totalpoll/emailcapture/services/action/services/register', [
			'mailchimp' => MailChimp::class
		] );
	}

	/**
	 * @param array $tabs
	 *
	 * @return array
	 */
	public function tab( $tabs ) {

		$tabs[ static::OPTION_NAME ] = [
			'label' => esc_html__( 'Email Capture', 'totalpoll' ),
			'icon'  => 'cloud-upload'
		];

		return $tabs;
	}

	/**
	 * @param string $tab
	 */
	public function render( $tab ) {
		if ( $tab === static::OPTION_NAME ) {

			foreach ( $this->services as $service => $class ) {
				add_action( 'totalpoll/emailcapture/services/action/settings', [ $class, 'renderSettings' ] );
			}

			include_once $this->getPath( '/views/index.php' );
		}
	}

	/**
	 * @param array $tabs
	 *
	 * @return array
	 */
	public function editor( $tabs ) {
		if ( $this->isEnabled() ) {
			$tabs['emailing'] = [
				'label' => esc_html__( 'Email Capture', 'totalpoll' ),
				'icon'  => 'cloud-upload',
				'file'  => $this->getPath( '/views/editor.php' )
			];
		}

		return $tabs;
	}

	/**
	 * @param array $fields
	 *
	 * @return array
	 */
	public function fields( $fields ) {
		if ( is_admin() && ! Misc::isDoingAjax() ) {
			return $fields;
		}

		if ( $this->isEnabled() && ! $this->hasField( $fields ) ) {
			$fields[] = [
				'uid'          => $this->emailField,
				'name'         => $this->emailField,
				'type'         => 'text',
				'label'        => esc_html__( 'Email', 'totalpoll' ),
				'defaultValue' => '',
				'options'      => [],
				'validations'  => [
					'filled' => [ 'enabled' => $this->getOption( 'field.required', false ) ],
					'email'  => [ 'enabled' => true ],
				],
			];
		}

		return $fields;
	}

	/**
	 * @param array $fields
	 *
	 * @return array
	 */
	public function formFields( $fields ) {

		if ( $this->getOption( 'consent.enabled', false ) ) {
			$fields[] = [
				'uid'         => $this->consentField,
				'type'        => 'checkbox',
				'label'       => false,
				'name'        => 'consent',
				'validations' => [
					'filled' => [ 'enabled' => true ],
				],
				'options'     => [
					'yes' => $this->getOption( 'consent.message', esc_html__( 'I agree to receive emails.', 'totalpoll' ) )
				]
			];
		}

		return $fields;
	}

	/**
	 * @return bool
	 */
	public function isEnabled() {
		return (bool) $this->getOption( 'enabled', false );
	}

	/**
	 * @param \TotalPoll\Entry\Model $entry
	 * @param Model $poll
	 */
	public function handle( $entry, $poll ) {
		if ( ! $this->isEnabled() || $poll->getSettingsItem( static::OPTION_NAME . '.disable', false ) ) {
			return;
		}

		/**
		 * @var AbstractService $service
		 */
		foreach ( $this->services as $serviceName => $serviceClass ) {

			if ( $this->isServiceEnabled( $serviceName ) ) {
				$service = new $serviceClass( $this );
				$service->handle( $entry, $poll );
			}
		}
	}

	/**
	 * @param array $fields
	 *
	 * @return bool
	 */
	protected function hasField( $fields ) {
		return in_array( $this->emailField, array_column( $fields, 'uid' ), true );
	}

	/**
	 * @param $serviceName
	 *
	 * @return bool
	 */
	protected function isServiceEnabled( $serviceName ) {
		return $this->getOption( "services.{$serviceName}.enabled", false );
	}

	/**
	 * @return string
	 */
	public function getEmailField() {
		return $this->emailField;
	}

	/**
	 * @return string
	 */
	public function getConsentField() {
		return $this->consentField;
	}
}
