<?php

namespace TotalPoll\Modules\Extensions\LivePolling;
! defined( 'ABSPATH' ) && exit();


use TotalPoll\Contracts\Poll\Model;
use WP_Error;

/**
 * Class Extension
 * @package TotalPoll\Modules\Extensions\LivePolling
 */
class Extension extends \TotalPoll\Modules\Extension {
	protected $root = __FILE__;

	/**
	 * Run the extension.
	 *
	 * @return mixed
	 */
	public function run() {
		add_action( 'rest_api_init', [ $this, 'registerRoutes' ] );
		add_filter( 'totalpoll/filters/render/output', [ $this, 'renderLiveResults' ], 10, 2 );
		add_filter( 'totalpoll/filters/admin/editor/integration/tabs', [ $this, 'tab' ] );
		add_action( 'totalpoll/actions/before/admin/editor/choices/toolbar/votes', [ $this, 'colorField' ] );
	}

	public function getLiveResultsUrl( $poll ) {
		$namespace = TotalPoll()->env( 'rest-namespace' );
! defined( 'ABSPATH' ) && exit();


		return get_rest_url( null, "{$namespace}/poll/{$poll->getId()}/live-polling" );
	}

	public function registerRoutes() {
		register_rest_route(
			TotalPoll()->env( 'rest-namespace' ), '/poll/(?P<id>\d+)/live-polling',
			[
				'methods'             => 'GET',
				'callback'            => [ $this, 'getLiveResults' ],
				'permission_callback' => '__return_true'
			]
		);
	}

	/**
	 * Get.
	 *
	 * @param \WP_REST_Request $restRequest
	 *
	 * @return array|WP_Error
	 */
	public function getLiveResults( \WP_REST_Request $restRequest ) {
		try {
			$status = get_post_status( (int) $restRequest['id'] );
			if ( $status === 'publish' ):
				$poll = TotalPoll( 'polls.repository' )->getById( (int) $restRequest['id'] );
			else:
				throw new \ErrorException( 'Invalid Poll ID', 404 );
			endif;

			return [
				'code'    => 200,
				'message' => '',
				'data'    => [
					'questions' => $this->getPollResults( $poll ),
					'status'    => 200,
				],
			];

		} catch ( \Exception $exception ) {
			return new WP_Error( $exception->getCode(), $exception->getMessage(), [ 'status' => 404 ] );
		}
	}

	/**
	 * @param       $rendered
	 * @param Model $poll
	 *
	 * @return string
	 */
	public function renderLiveResults( $rendered, $poll ) {
		if ( strpos( $poll->getScreen(), 'live' ) !== false ):
			$context     = explode( ':', $poll->getScreen() );
			$chartType   = empty( $context[1] ) ? 'doughnut' : $context[1];
			$chartLegend = empty( $context[2] ) ? false : (bool) $context[2];
			$this->assets();
			$rendered = $this->getResultsView( $poll, $chartType, $chartLegend );
		endif;

		return $rendered;
	}

	/**
	 * @param Model $poll
	 *
	 * @return string
	 */
	public function getResultsView( $poll, $chartType = 'doughnut', $chartLegend = true ) {
		ob_start();

		$settings = $this->getPollResults( $poll );
		$url      = $this->getLiveResultsUrl( $poll );

		include $this->getPath( 'views/results.php' );

		return ob_get_clean();
	}

	public function getPollResults( $poll ) {
		$questions = [];

		foreach ( $poll->getQuestionsForResults() as $questionIndex => $question ):
			$questions[ $questionIndex ] = [];
			// Question
			$questions[ $questionIndex ]['content'] = $question['content'];

			foreach ( $question['choices'] as $choice ):
				if ( empty( $choice['visibility'] ) ):
					continue;
				endif;
				// Choice
				$questions[ $questionIndex ]['choices'][ $choice['uid'] ] = [
					'label'          => $choice['label'],
					'votes'          => $choice['votes'],
					'color'          => empty( $choice['color'] ) ? null : $choice['color'],
					'votesFormatted' => $poll->getChoiceVotesFormatted( $choice['uid'] ),
				];
			endforeach;
		endforeach;

		return $questions;
	}

	public function assets() {
		wp_enqueue_script( 'chart.js', TotalPoll()->env( 'url' ) . "assets/dist/scripts/vendor/Chart.min.js", [ 'jquery' ] );
		wp_enqueue_script(
			'totalpoll-live-polling',
			$this->getUrl( 'assets/scripts/live-polling.js' ),
			[ 'chart.js' ]
		);
		wp_enqueue_style(
			'totalpoll-live-polling',
			$this->getUrl( 'assets/styles/live-polling.css' )
		);
	}

	public function tab( $tabs ) {
		$tabs['live-polling'] = [
			'label'       => esc_html__( 'Live Results', 'totalpoll' ),
			'description' => esc_html__( 'Results in realtime', 'totalpoll' ),
			'icon'        => 'update',
			'file'        => $this->getPath( 'views/tab.php' ),
		];

		return $tabs;
	}

	public function colorField() {
		?>
        <input
                class="totalpoll-form-field-input"
                type="color"
                style="margin-right: 10px"
                id="{{$ctrl.prefix('color')}}"
                name="{{$ctrl.prefix('color')}}"
                ng-model="$ctrl.item.color">
		<?php
	}
}
