<?php /** @noinspection PhpIncludeInspection */

namespace TotalPoll\Modules\Extensions\VoteConfirmation;
! defined( 'ABSPATH' ) && exit();


use TotalPoll\Log\Model as LogModel;
use TotalPoll\Modules\Extensions\VoteConfirmation\Concerns\EmailConfirmationHelper;
use TotalPoll\Notification\Mail;
use TotalPoll\Poll\Model as PollModel;
use TotalPollVendors\TotalCore\Helpers\Arrays;
use TotalPollVendors\TotalCore\Helpers\Misc;
use TotalPollVendors\TotalCore\Helpers\Strings;
use TotalPollVendors\TotalCore\Http\Request;

/**
 * Class Extension
 * @package TotalPoll\Modules\Extensions\VoteConfirmation
 */
class Extension extends \TotalPoll\Modules\Extension {
	const OPTION_NAME = 'vote_confirmation';
	const FIELD_NAME = 'email';

	use EmailConfirmationHelper;

	/**
	 * @var string
	 */
	protected $root = __FILE__;

	/**
	 * @var array
	 */
	protected $defaultsMessages = [];

	/**
	 * Extension constructor.
	 *
	 */
	public function __construct() {
		parent::__construct( TotalPoll()->option( self::OPTION_NAME, [] ) );

		$this->defaultsMessages = [
			'subject'             => esc_html__( 'Please confirm your vote', 'totalpoll' ),
			'body'                => esc_html__( 'Thank you for voting in the poll, please confirm the vote by clicking on the link below', 'totalpoll' ),
			'message'             => esc_html__( 'We’ve sent a confirmation email to your inbox, please click on the link to confirm your vote.', 'totalpoll' ),
			'confirmation'        => esc_html__( 'Thank you, your vote has been confirmed.', 'totalpoll' ),
			'already_confirmed'   => esc_html__( 'Vote already confirmed', 'totalpoll' ),
			'cannot_confirm'      => esc_html__( 'Vote cannot be confirmed', 'totalpoll' ),
			'confirmation_button' => esc_html__( 'Confirm my vote', 'totalpoll' ),
		];
	}

	/**
	 * @return mixed|void
	 */
	public function run() {
		// Admin
		add_filter( 'totalpoll/filters/admin/options/tabs', [ $this, 'tab' ] );
		add_filter( 'totalpoll/filters/admin/editor/settings/tabs', [ $this, 'editor' ] );
		add_action( 'totalpoll/actions/after/admin/options/tabs/content', [ $this, 'render' ] );
		add_filter( 'totalpoll/filters/admin/entries/export/fields', [ $this, 'export' ] );

		// Front
		add_filter( 'totalpoll/filters/poll/settings-item/fields', [ $this, 'fields' ], 10, 2 );
		add_filter( 'totalpoll/filters/poll/choices/count', [ $this, 'disableVoteCount' ], 10, 2 );
		add_action( 'totalpoll/actions/request/vote_confirmation', [ $this, 'confirmVote' ] );

	}

	/**
	 * @param array $tabs
	 *
	 * @return array
	 */
	public function tab( $tabs ) {

		$tabs[ static::OPTION_NAME ] = [
			'label' => esc_html__( 'Vote Confirmation', 'totalpoll' ),
			'icon'  => 'buddicons-pm'
		];

		return $tabs;
	}

	/**
	 * @param string $tab
	 */
	public function render( $tab ) {
		if ( $tab === static::OPTION_NAME ) {
			if ( TotalPoll( 'http.request' )->request( 'resubmit-confirmations' ) ) {
				$resent = $this->resendConfirmationEmails();
			}

			include_once $this->getPath( '/views/options.php' );
		}
	}

	/**
	 * @param array $tabs
	 *
	 * @return array
	 */
	public function editor( $tabs ) {
		if ( $this->isEnabled() ) {
			$tabs[ static::OPTION_NAME ] = [
				'label' => esc_html__( 'Vote Confirmation', 'totalpoll' ),
				'icon'  => 'buddicons-pm',
				'file'  => $this->getPath( '/views/editor.php' )
			];
		}

		return $tabs;
	}

	/**
	 * @param array $fields
	 *
	 * @param array $poll
	 *
	 * @return array
	 */
	public function fields( $fields, $poll ) {

		if ( is_admin() && ! Misc::isDoingAjax() ) {
			return $fields;
		}

		if ( $this->isEnabled() && ! $this->isConfirmationExcluded( $poll ) && ! $this->hasField( $fields ) ) {
			$fields[] = [
				'uid'          => static::FIELD_NAME,
				'name'         => static::FIELD_NAME,
				'type'         => 'text',
				'label'        => $this->getFieldLabel(),
				'defaultValue' => '',
				'options'      => [],
				'validations'  => [
					'filled' => [ 'enabled' => true ],
					'email'  => [ 'enabled' => true ],
					'unique' => [ 'enabled' => $this->isUnique() ]
				],
			];
		}

		return $fields;
	}

	/**
	 * @param array $fields
	 *
	 * @return array
	 */
	public function export( $fields ) {
		if ( $this->isEnabled() && ! $this->hasField( $fields ) ) {
			$fields[] = [
				'uid'          => static::FIELD_NAME,
				'name'         => static::FIELD_NAME,
				'type'         => 'text',
				'label'        => esc_html__( 'Email', 'totalpoll' ),
				'defaultValue' => '',
				'options'      => [],
				'validations'  => [
					'filled' => [ 'enabled' => true ],
					'email'  => [ 'enabled' => true ],
					'unique' => [ 'enabled' => $this->isUnique() ]
				],
			];
		}

		return $fields;
	}


	/**
	 * @param $choices
	 * @param PollModel $poll
	 *
	 * @return bool
	 */
	public function disableVoteCount( $choices, $poll ) {
		if ( $this->isEnabled() && ! $this->isConfirmationExcluded( $poll->getSettings() ) ) {

			foreach ( $choices as $index => $choice ) {
				$choices[ $index ]['votes'] = 0;
			}

			add_filter( 'totalpoll/filters/poll/command/log/attributes', [ $this, 'sendConfirmationEmail' ], 10, 2 );
		}

		return $choices;
	}

	public function resendConfirmationEmails() {
		$database      = TotalPoll( 'database' );
		$databaseTable = TotalPoll()->env( 'db.tables.log' );
		$sql           = "SELECT * FROM {$databaseTable} WHERE (action= %s) AND (status IN (%s, %s))";
		$query         = $database->prepare( $sql, 'vote', 'undelivered', 'pending' );
		$logs          = $database->get_results( $query, ARRAY_A );

		foreach ( $logs as $log ) {
			$logModel = new LogModel( $log );
			$poll     = $logModel->getPoll();

			$email = $logModel->getEntry()->getField( static::FIELD_NAME );
			$token = $logModel->getDetails()['confirmation_token'];

			$subject = $this->getSubject();
			$message = $this->getBody();

			$choices = [];

			foreach ( $logModel->getChoices() as $choice ) {
				$choices[] = esc_html( $poll->getChoice( $choice )['label'] );
			}

			$url = add_query_arg( [
				'totalpoll' => [
					'action' => 'vote_confirmation',
					'token'  => $token
				],
			], $poll->getPermalink() );

			$vars = [
				'poll'    => $poll->getTitle(),
				'user'    => $this->getUserName(),
				'choices' => implode( ', ', $choices )
			];

			$body = $this->renderTemplate( [
				'url'     => $url,
				'message' => Strings::template( $message, $vars ),
				'button'  => $this->getOption( 'confirmation_button', $this->getDefaultMessage( 'confirmation_button' ) )
			] );


			$mail = new Mail();
			$sent = $mail->setTo( $email )
			             ->setSubject( Strings::template( $subject, $vars ) )
			             ->setBody( $body )
			             ->send();

			TotalPoll( 'log.repository' )->update( $logModel->getId(), [ 'status' => $sent ? 'pending' : 'undelivered' ] );
		}

		return 1;
	}

	/**
	 * @param array $logAttributes
	 * @param PollModel $poll
	 *
	 * @return array
	 */
	public function sendConfirmationEmail( array $logAttributes, PollModel $poll ) {
		$email = Arrays::getDotNotation( $poll->getForm()->toArray(), static::FIELD_NAME );
		$token = sha1( wp_rand( 40 ) );

		$logAttributes['details']['confirmation_token'] = $token;
		$logAttributes['status']                        = 'pending';

		$subject = $this->getSubject();
		$message = $this->getBody();

		$choices = [];

		foreach ( $poll->getReceivedChoices() as $choice ) {
			$choices[] = esc_html( $choice['label'] );
		}

		$url = $poll->getUrl( [
			'totalpoll' => [
				'action' => 'vote_confirmation',
				'token'  => $token
			]
		] );

		$vars = [
			'poll'    => $poll->getTitle(),
			'user'    => $this->getUserName(),
			'choices' => implode( ', ', $choices )
		];

		$body = $this->renderTemplate( [
			'url'     => $url,
			'message' => Strings::template( $message, $vars ),
			'button'  => $this->getOption( 'confirmation_button', $this->getDefaultMessage( 'confirmation_button' ) )
		] );


		$mail = new Mail();
		$sent = $mail->setTo( $email )
		             ->setSubject( Strings::template( $subject, $vars ) )
		             ->setBody( $body )
		             ->send();

		$logAttributes['status'] = $sent ? 'pending' : 'undelivered';

		$this->notifyUser( $this->getMessage() );

		return $logAttributes;
	}


	public function confirmVote() {
		/**
		 * @var Request $request
		 */
		$request = TotalPoll( 'http.request' );
		$token   = $request->query( 'totalpoll.token' );
		$log     = $this->getLogByToken( $token );

		add_filter( 'totalpoll/filters/poll/settings-item/content.results.below', function ( $content ) {
			$script = "<script>window.history.replaceState({}, document.title, window.location.href.split('?')[0])</script>";

			return $content . $script;
		} );

		if ( ! $log ) {
			$message = $this->getOption( 'cannot_confirm', $this->getDefaultMessage( 'cannot_confirm' ) );
			$this->notifyUser( $message, 'error' );

			return;
		}

		$poll = $log->getPoll();
		$poll->setScreen( PollModel::RESULTS_SCREEN );

		if ( $log->getStatus() === 'accepted' ) {
			$message = $this->getOption( 'already_confirmed', $this->getDefaultMessage( 'already_confirmed' ) );
			$this->notifyUser( $message, 'warning' );

			return;
		}

		$choices = [];

		foreach ( $log->getChoices() as $choice ) {
			$choices[] = [
				'uid'   => $choice,
				'votes' => 1
			];
		}

		TotalPoll( 'polls.repository' )->incrementVotes( $log->getPollId(), $choices );
		TotalPoll( 'log.repository' )->update( $log->getId(), [ 'status' => 'accepted' ] );

		$this->notifyUser( $this->getConfirmation(), 'success' );
	}

	/**
	 * @param string $token
	 *
	 * @return false|LogModel
	 */
	protected function getLogByToken( $token ) {
		if ( empty( $token ) ) {
			return false;
		}

		$database      = TotalPoll( 'database' );
		$databaseTable = TotalPoll()->env( 'db.tables.log' );
		$search        = sprintf( '%%"%s":"%s"%%', 'confirmation_token', $token );
		$sql           = "SELECT * FROM {$databaseTable} WHERE action= %s AND details LIKE %s";
		$query         = $database->prepare( $sql, 'vote', $search );
		$log           = $database->get_row( $query, ARRAY_A );

		if ( ! $log ) {
			return false;
		}

		return new LogModel( $log );
	}

	public function notifyUser( $text, $status = 'success' ) {
		add_filter( 'totalpoll/filters/poll/settings-item/content.results.above', function ( $content ) use ( $text, $status ) {
			$message = sprintf( '<div class="totalpoll-message totalpoll-message-%s">%s</div>', $status, $text );

			return $message . $content;
		} );
	}

}
